import os
import sys
import time
import shutil
import pathlib
import imp
from yaplee.server import Server
from distutils.dir_util import copy_tree
import subprocess
from bs4 import BeautifulSoup

class AppStarter:
    def __init__(self, argv):
        self.__TRUE_PAGE_FILES = ['__init__.py', 'config.py', 'controller.py']
        self.__TRUE_PAGE_FOLDERS = ['static', 'template']
        self.__DAMAGED_APPS = []
        self.__LOADED_PAGES = []
        self.__ERRORS = 0

        self.user_path = os.getcwd()
        self.module_path = str(pathlib.Path(__file__).resolve().parent)
        self.argv = [a.lower() for a in argv]
        self.argv.pop(0)

        self.ScanApplication()

        if(self.argv == []):
            print('Usage: python application.py <command>')
            sys.exit(0)

        elif(self.argv[0] == 'run'):
            print('Loading application files...')
            time.sleep(0.5)
            try:
                self.RunServer()
            except:
                print('Permission error to create new page')
                if(os.name != 'nt'):
                    print('Tip: Execute commands with sudo')


        elif(self.argv[0] == 'newpage'):
            if(len(self.argv) == 1):
                print('Usage: python application.py newpage <page name>')
            else:
                print('Creating page...')
                time.sleep(0.5)
                self.NewPage(page_name=self.argv[1])
        else:
            self.UnknownCommand()



    def ScanApplication(self):
        for page_r, page_d, page_f in os.walk(os.path.join(self.user_path), 'pages'):
            for page in page_d:
                if(page in self.GetApplicationConfig()['APP_PAGES']):
                    for r,d,f in os.walk(os.path.join(self.user_path, 'pages', str(page))):
                        if(r == os.path.join(self.user_path, 'pages', str(page))):
                            for TAF in self.__TRUE_PAGE_FOLDERS:
                                if(TAF not in d):
                                    self.__DAMAGED_APPS.append(page); self.__ERRORS += 1
                                    continue
                                else:
                                    if(page not in self.__LOADED_PAGES):
                                        self.__LOADED_PAGES.append(page)

                            for TAF in self.__TRUE_PAGE_FILES:
                                if(TAF not in f and page not in self.__DAMAGED_APPS):
                                    self.__DAMAGED_APPS.append(page); self.__ERRORS += 1
                                    continue
                                else:
                                    if(page not in self.__LOADED_PAGES):
                                        self.__LOADED_PAGES.append(page)

        if(len(self.__DAMAGED_APPS) > 2):
            self.not_loaded_pages = str(', '.join(self.__DAMAGED_APPS[:2]))+' and '+str(len(self.__DAMAGED_APPS)-2)+' more pages'
        else:
            self.not_loaded_pages = str(', '.join(self.__DAMAGED_APPS))

    def GetApplicationConfig(self):
        config = imp.load_source('application.config', os.path.join(self.user_path, 'config.py'))
        return_it = {'APP_PAGES':config.APP_PAGES, 'DEVELOPMENT_SERVER_PORT':config.DEVELOPMENT_SERVER_PORT}
        return return_it

    def Load_Controllers(self, page, server_path):
        try:
            config = imp.load_source('page.config', os.path.join(self.user_path, 'pages', page, 'config.py'))
            app = config.page
            for key, value in app.GetRegisterList().items():
                for i in value:
                    if(i != ''):
                        os.mkdir(server_path+'/'+i)
                section = app.GetSections()[key]
                for url in value:
                    for template in section['templates']:
                        sub_path = server_path+'/'+url if url != '' else server_path
                        shutil.copy(
                            'pages/'+section['page_name']+'/template/'+template,
                            sub_path+'/'+template
                        )
                    with open(sub_path+'/'+template, 'r+') as template_file:
                        template_content = template_file.read()
                        soup = BeautifulSoup(template_content, 'html.parser')

                    for static in section['statics']:
                        link_style = soup.new_tag('link')
                        link_style['rel'] = 'stylesheet'
                        link_style['href'] = static
                        soup.head.append(link_style)

                    with open(sub_path+'/'+template, 'w+') as template_file:
                        template_file.write(soup.prettify())

                    for static in section['statics']:
                        shutil.copy(
                            'pages/'+section['page_name']+'/static/'+static,
                            sub_path+'/'+static
                        )
        except:
            pass


    def RunServer(self):
        print(
            'No error found in the application'
            if self.__DAMAGED_APPS == [] else
            str(self.__ERRORS)+' errors were found in the application, ('+str(self.not_loaded_pages)+') were not loaded.'
        )
        print('Note: Use CTRL+C to quit development server')
        server = Server(int(self.GetApplicationConfig()['DEVELOPMENT_SERVER_PORT']))
        try:
            server.CheckSystem()
            if(self.__LOADED_PAGES == []):
                print('There is no page to run! Stopping WebServer...')
                sys.exit()
            for page in self.__LOADED_PAGES:
                self.Load_Controllers(page, server.server_foldername) # Load Pages
            subprocess.run(
            'python -m http.server '+str(self.GetApplicationConfig()['DEVELOPMENT_SERVER_PORT'])+' --bind 127.0.0.1 --directory "'+str(os.path.join(self.user_path, 'pages', '.server'))+'"',
            shell=True
            )
        except KeyboardInterrupt:
            server.Stop()

    def UnknownCommand(self):
        print('Unknown command : '+str(self.argv[0]).title())
        sys.exit(0)

    def RemoveDir(self, dir_):
        dir_ = dir_.replace('\\', '/')
        for r,d,f in os.walk(dir_):
            for i in d:
                self.RemoveDir(r+'/'+i)
            for i in f:
                i = i.replace('/', '')
                os.unlink(r+'/'+i)
            for i in d:
                os.rmdir(r+'/'+i)

    def NewPage(self, page_name):
        if(os.path.isdir(os.path.join(self.user_path, 'pages', str(page_name)))):
            print('There is already a page with this name')
            sys.exit(0)
        try:
            os.mkdir(os.path.join(self.user_path, 'pages', str(page_name)))
            copy_tree(
                os.path.join(self.module_path, 'app', 'Blank'),
                os.path.join(self.user_path, 'pages', str(page_name))
            )
            with open(os.path.join(self.module_path, 'app', 'Blank', 'config.py'), 'r+') as config_text:
                with open(os.path.join(self.user_path, 'pages', str(page_name), 'config.py'), 'w+') as config:
                    config.write(config_text.read().replace('[PAGE_NAME]', str(page_name)))
            print(str(page_name).title()+' page created')
        except:
            self.RemoveDir(os.path.join(self.user_path, 'pages', str(page_name)))
            print('Permission error to create new page')
            if(os.name != 'nt'):
                print('Tip: Execute commands with sudo')



import sys

def yaplee_app():
    try:
        from yaplee.core import AppStarter
    except ImportError as as_exec:
        raise ImportError(
            "There is a problem importing yaplee, "
            "Please make sure yaplee is installed"
        ) from as_exec
    AppStarter(sys.argv)

if __name__ == '__main__':
    yaplee_app()


from distutils.dir_util import copy_tree
from yaplee.defaults import HELLOWORLD_CSS, HELLOWORLD_TEMPLATE
import shutil
import time
import os

class Manager:
    def __init__(self, user_path, module_path, argv):
        self.user_path = user_path
        self.module_path = module_path
        self.argv = argv

    def clearCache(self, path):
        for directories, subfolder, files in os.walk(path):
            if os.path.isdir(directories):
                if directories[::-1][:11][::-1] == '__pycache__':
                                shutil.rmtree(directories)
    
    def Create(self, app_name):
        copy_tree(
            os.path.join(str(self.module_path), 'app', 'HelloWorld'),
            os.path.join(str(self.user_path), str(app_name))
        )
        time.sleep(0.5)
        self.clearCache(os.path.join(str(self.user_path), str(app_name)))
        os.unlink(os.path.join(str(self.user_path), str(app_name), '__init__.py'))
        with open(os.path.join(str(self.user_path), str(app_name), 'pages', 'HelloWorld', 'static', 'style.css'), 'w+') as style:
            style.write(HELLOWORLD_CSS)
        with open(os.path.join(str(self.user_path), str(app_name), 'pages', 'HelloWorld', 'template', 'index.html'), 'w+') as templtae:
            templtae.write(HELLOWORLD_TEMPLATE)


from yaplee.template import PageController as Controller

page = Controller(page_name='HelloWorld')

# Your page sections are here
SECTIONS = [
    page.Section(name='Main')
]

# Your page template are here
TEMPLATES = [
    page.Template(path='index.html', section='Main')
]

# Your page template static files are here
STATICS = [
    page.Static(path='style.css', section='Main')
]

# Your sections want to register are here
page.Register('Main', url='')


HELLOWORLD_TEMPLATE = '''
<!DOCTYPE html>
<html lang="en" >
<head>
  <meta charset="UTF-8">
  <title>Hello, World!</title>
  <meta name="viewport" content="width=device-width, initial-scale=1">
  <link rel='stylesheet' href='https://fonts.googleapis.com/css?family=Lato:100,300'>

</head>
<body>
<div id="main">
    	<div class="fof">
        		<h1>Hello, World!</h1>
        		<h4>
        			This is a default page from Yaplee<br>
        			This page is located in the project "pages" folder<br>
        			<a href="#">Click here to view the full Yaplee documents</a>

        		</h4>
    	</div>
</div>
</body>
</html>
'''

HELLOWORLD_CSS = '''
*{
    transition: all 0.6s;
}

html {
    height: 100%;
}

body{
    font-family: 'Lato', sans-serif;
    color: #888;
    margin: 0;
}

#main{
    display: table;
    width: 100%;
    height: 100vh;
    text-align: center;
}

.fof{
	  display: table-cell;
	  vertical-align: middle;
}

.fof h1{
	  font-size: 50px;
	  display: inline-block;
	  padding-right: 12px;
	  animation: type .5s alternate infinite;
}

@keyframes type{
	  from{box-shadow: inset -3px 0px 0px #888;}
	  to{box-shadow: inset -3px 0px 0px transparent;}
}'''


from yaplee.app_manager import Manager
import os
import pathlib
import sys

class Yaplee_Main:
    def __init__(self):
        self.user_path = os.getcwd()
        self.module_path = str(pathlib.Path(__file__).resolve().parent)
        self.argv = [argv.lower() for argv in sys.argv]
        self.argv.pop(0)

        self.manager = Manager(
            self.user_path,
            self.module_path,
            self.argv
        )

        if(self.argv == []):
            self.NoneCommand()

        elif(self.argv[0] == 'create'):
            self.Create()

        else:
            self.UnknownCommand()

    def NoneCommand(self):
        print('Usage: yaplee <command>\nUse help to show all commands')
        sys.exit(0)
    
    def GetArgv(self, index):
        try:
            return self.argv[index]
        except:
            return False
    
    def UnknownCommand(self):
        print('Unknown command : '+str(self.argv[0]).title())
        sys.exit(0)
    
    def Create(self):
        if(len(self.argv) == 1):
            print('Please enter the name of the Yaplee application')
            print('Usage: yaplee create <application name>')
            sys.exit(0)
        
        if(os.path.isdir(self.argv[1])):
            print('Warning: [{}] already exists'.format(str(os.path.join(self.user_path, self.argv[1]))))
            sys.exit(0)

        print('Starting a yaplee application...')
        self.manager.Create(self.argv[1])
        print(self.argv[1].title()+' application was started')
        if(self.GetArgv(2) in ['-c', '--vscode']):
            os.system('code '+str(self.argv[1]))
        sys.exit(0)


from setuptools import find_packages, setup
import os

with open(os.path.join(os.path.dirname(__file__), 'yaplee', '__init__.py')) as init_file:
    for line in init_file.read().splitlines():
        if(line.lower().startswith('__version__')):
            exec(line)

setup(
    name="Yaplee",
    version=str(__version__),
    description="Yaplee is a fun and simple python framework to build user interfaces on web pages",
    url="https://github.com/YapleeProject",
    author='Matin Najafi',
    author_email="matinnajafi.dev@gmail.com",
    license="MIT",
    include_package_data=True,
    long_description='Yaplee is a Powerful, Fun and Open Souce MIT-Licenced project for front-end programming in Python. Yaplee framework is very simple and does not have complicated details.',
    long_description_content_type="text/markdown",
    install_requires=['bs4'],
    packages=find_packages(),
    entry_points={
        'console_scripts': [
            'yaplee = yaplee.main:Yaplee_Main'
        ]
    },
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    python_requires='>=3.6',
    zip_safe=False
)


import subprocess
import os
import socket
import sys

class Server:
    def __init__(self, server_port):
        self.server_foldername = 'pages/.server'
        self.port = server_port

    def RemoveDir(self, dir_):
        dir_ = dir_.replace('\\', '/')
        for r,d,f in os.walk(dir_):
            for i in d:
                self.RemoveDir(r+'/'+i)
            for i in f:
                i = i.replace('/', '')
                os.unlink(r+'/'+i)
            for i in d:
                os.rmdir(r+'/'+i)

    def CheckSystem(self):
        a_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        local_connection = ('127.0.0.1', self.port)
        CHECK_PORT = a_socket.connect_ex(local_connection)
        a_socket.close()
        if(CHECK_PORT != 0):
            if(os.path.isdir(self.server_foldername)):
                self.RemoveDir(self.server_foldername)
                os.rmdir(self.server_foldername)
            os.mkdir(self.server_foldername)
            if(os.name == 'nt'):
                import win32api
                import win32con
                win32api.SetFileAttributes(self.server_foldername, win32con.FILE_ATTRIBUTE_HIDDEN)
        else:
            print('Server starting error on port '+str(self.port)+', Is the yaplee server started by another file?')
            sys.exit(0)

    def Stop(self):
        if(os.path.isdir(self.server_foldername)):
            self.RemoveDir(self.server_foldername)
            os.rmdir(self.server_foldername)



import sys
import os
import pathlib

class PageController:
    def __init__(self, page_name):
        self.__PAGE = page_name
        self.__SECTIONS = {}
        self.__URLS = {}
        self.user_path = os.getcwd()
        self.module_path = str(pathlib.Path(__file__).resolve().parent)

    def Section(self, name):
        if(name in self.__SECTIONS):
            print(str(name)+' section can\'t be defined more than once!')
            sys.exit(0)

        self.__SECTIONS[name] = {
            'page_name':self.__PAGE,
            'templates':[],
            'statics':[]
            }
        
        self.__URLS[name] = []
    
    def Template(self, path, section):
        AVALIABLE_TEMPLATES = []

        for root, directory, file_ in os.walk(os.path.join(self.user_path, 'pages', str(self.__PAGE), 'template')):
            for f in file_:
                if(f != '__init__.py'):
                    AVALIABLE_TEMPLATES.append(f)
        
        if(path not in AVALIABLE_TEMPLATES):
            print('Template ['+self.__PAGE+':'+path+'] not found!')
            sys.exit(0)

        if(section not in self.__SECTIONS):
            print('Section ['+self.__PAGE+':'+section+'] not found!')
            sys.exit(0)
        
        self.__SECTIONS[section]['templates'].append(path)
        
    def Static(self, path, section):
        AVALIABLE_STATICS = []

        for root, directory, file_ in os.walk(os.path.join(self.user_path, 'pages', str(self.__PAGE), 'static')):
            for f in file_:
                if(f != '__init__.py'):
                    AVALIABLE_STATICS.append(f)
        
        if(path not in AVALIABLE_STATICS):
            print('Static file ['+self.__PAGE+':'+path+'] not found!')
            sys.exit(0)

        if(section not in self.__SECTIONS):
            print('Section ['+self.__PAGE+':'+section+'] not found!')
            sys.exit(0)
        
        self.__SECTIONS[section]['statics'].append(path)
    
    def Register(self, section, url):
        self.__URLS[section].append(url)
    
    def GetRegisterList(self):
        return self.__URLS
    
    def GetSections(self):
        return self.__SECTIONS